------------------------------------------------------------------------------
-- generate_fork.lua --
--   create generator modelling fork between philosophers i1 and i2
--   for the WODES'08 "dining philosophers" benchmark example.
--
-- parameters: i1 - index of philosopher situated to the left
--             i2 - index of philosopher situated to the right
-- 
-- return: generator model of fork
------------------------------------------------------------------------------

-- *** FUNCTION Fork: returns fork between philosophers i1 and i2
function Fork(i1,i2)
    
    fork=Generator.New()
    
    fork:Name(string.format('Fork_n=%d_n=%d',i1,i2))
    
    -- catch invalid inputs
    if(i1==i2) then
        print('ERROR in generate_fork.lua: i1 and i2 have to be different - empty result.')
        return fork
    end
    
    -- create init state i1_i2A ("Available")
    stateA=fork:InsInitState(string.format('%d_%dA',i1,i2))
    fork:SetMarkedState(stateA)
  
    -- create state i1_i2U ("in Use")
    stateU=fork:InsState(string.format('%d_%dU',i1,i2))
  
    -- transitions "philospher i1 picks right fork" (i1fr) and
    -- "philospher i2 picks left fork" (i2fl) lead to state
    -- i1_i2_U, where i2fl is uncontrollable for i2 even
    event = fork:InsControllableEvent(string.format('%dfr',i1))
    fork:SetTransition(stateA,event,stateU)
    
    if((i2 % 2)==0) then -- if i2 is even, set i2fl uncontrollable
	    event = fork:InsUncontrollableEvent(string.format('%dfl',i2))
    else
        event = fork:InsControllableEvent(string.format('%dfl',i2))
    end
    fork:SetTransition(stateA,event,stateU)
    
    -- transitions "philospher drops forks" (i1f,i2f) lead
	-- back to state i1_i2_A.
    event = fork:InsControllableEvent(string.format('%df',i1))
    fork:SetTransition(stateU,event,stateA)
    event = fork:InsControllableEvent(string.format('%df',i2))
    fork:SetTransition(stateU,event,stateA)

    return fork
end
-- *** ENDOF FUNCTION fork